@@ -397,7 +397,7 @@ class ProductQuantitiesByWarehouse(ModelSQL, ModelView):
             key = trans_context['product_template']
         warehouse_id = trans_context.get('warehouse')
 
-        dates = sorted(l.date for l in lines)
+        dates = sorted({l.date for l in lines})
         quantities = {}
         date_start = None
         for date in dates:
